@@ -9,6 +9,8 @@ These include:
 * heat index
 * windchill
 """
+import contextlib
+from itertools import product
 import warnings
 
 import numpy as np
@@ -16,18 +18,19 @@ from scipy.ndimage import gaussian_filter
 
 from .. import constants as mpconsts
 from ..package_tools import Exporter
-from ..units import atleast_1d, check_units, masked_array, units
-from ..xarray import preprocess_xarray
+from ..units import check_units, masked_array, units
+from ..xarray import preprocess_and_wrap
 
 exporter = Exporter(globals())
 
 # The following variables are constants for a standard atmosphere
 t0 = 288. * units.kelvin
 p0 = 1013.25 * units.hPa
+gamma = 6.5 * units('K/km')
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='u')
 @check_units('[speed]', '[speed]')
 def wind_speed(u, v):
     r"""Compute the wind speed from u and v-components.
@@ -42,7 +45,7 @@ def wind_speed(u, v):
     Returns
     -------
     wind speed: `pint.Quantity`
-        The speed of the wind
+        Speed of the wind
 
     See Also
     --------
@@ -54,7 +57,7 @@ def wind_speed(u, v):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='u')
 @check_units('[speed]', '[speed]')
 def wind_direction(u, v, convention='from'):
     r"""Compute the wind direction from u and v-components.
@@ -66,14 +69,14 @@ def wind_direction(u, v, convention='from'):
     v : `pint.Quantity`
         Wind component in the Y (North-South) direction
     convention : str
-        Convention to return direction. 'from' returns the direction the wind is coming from
-        (meteorological convention). 'to' returns the direction the wind is going towards
-        (oceanographic convention). Default is 'from'.
+        Convention to return direction; 'from' returns the direction the wind is coming from
+        (meteorological convention), 'to' returns the direction the wind is going towards
+        (oceanographic convention), default is 'from'.
 
     Returns
     -------
     direction: `pint.Quantity`
-        The direction of the wind in interval [0, 360] degrees, with 360 being North, with the
+        The direction of the wind in intervals [0, 360] degrees, with 360 being North,
         direction defined by the convention kwarg.
 
     See Also
@@ -88,7 +91,7 @@ def wind_direction(u, v, convention='from'):
     """
     wdir = 90. * units.deg - np.arctan2(-v, -u)
     origshape = wdir.shape
-    wdir = atleast_1d(wdir)
+    wdir = np.atleast_1d(wdir)
 
     # Handle oceanographic convection
     if convention == 'to':
@@ -108,17 +111,17 @@ def wind_direction(u, v, convention='from'):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like=('speed', 'speed'))
 @check_units('[speed]')
-def wind_components(speed, wdir):
+def wind_components(speed, wind_direction):
     r"""Calculate the U, V wind vector components from the speed and direction.
 
     Parameters
     ----------
     speed : `pint.Quantity`
-        The wind speed (magnitude)
-    wdir : `pint.Quantity`
-        The wind direction, specified as the direction from which the wind is
+        Wind speed (magnitude)
+    wind_direction : `pint.Quantity`
+        Wind direction, specified as the direction from which the wind is
         blowing (0-2 pi radians or 0-360 degrees), with 360 degrees being North.
 
     Returns
@@ -138,15 +141,18 @@ def wind_components(speed, wdir):
     >>> metpy.calc.wind_components(10. * units('m/s'), 225. * units.deg)
      (<Quantity(7.07106781, 'meter / second')>, <Quantity(7.07106781, 'meter / second')>)
 
+    .. versionchanged:: 1.0
+       Renamed ``wdir`` parameter to ``wind_direction``
+
     """
-    wdir = _check_radians(wdir, max_radians=4 * np.pi)
-    u = -speed * np.sin(wdir)
-    v = -speed * np.cos(wdir)
+    wind_direction = _check_radians(wind_direction, max_radians=4 * np.pi)
+    u = -speed * np.sin(wind_direction)
+    v = -speed * np.cos(wind_direction)
     return u, v
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='temperature')
 @check_units(temperature='[temperature]', speed='[speed]')
 def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
     r"""Calculate the Wind Chill Temperature Index (WCTI).
@@ -157,14 +163,14 @@ def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
     Specifically, these formulas assume that wind speed is measured at
     10m.  If, instead, the speeds are measured at face level, the winds
     need to be multiplied by a factor of 1.5 (this can be done by specifying
-    `face_level_winds` as `True`.)
+    `face_level_winds` as `True`).
 
     Parameters
     ----------
     temperature : `pint.Quantity`
-        The air temperature
+        Air temperature
     speed : `pint.Quantity`
-        The wind speed at 10m.  If instead the winds are at face level,
+        Wind speed at 10m. If instead the winds are at face level,
         `face_level_winds` should be set to `True` and the 1.5 multiplicative
         correction will be applied automatically.
     face_level_winds : bool, optional
@@ -180,11 +186,11 @@ def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
     Returns
     -------
     `pint.Quantity`
-        The corresponding Wind Chill Temperature Index value(s)
+        Corresponding Wind Chill Temperature Index value(s)
 
     See Also
     --------
-    heat_index
+    heat_index, apparent_temperature
 
     """
     # Correct for lower height measurement of winds if necessary
@@ -208,9 +214,9 @@ def windchill(temperature, speed, face_level_winds=False, mask_undefined=True):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='temperature')
 @check_units('[temperature]')
-def heat_index(temperature, rh, mask_undefined=True):
+def heat_index(temperature, relative_humidity, mask_undefined=True):
     r"""Calculate the Heat Index from the current temperature and relative humidity.
 
     The implementation uses the formula outlined in [Rothfusz1990]_, which is a
@@ -223,14 +229,14 @@ def heat_index(temperature, rh, mask_undefined=True):
     ----------
     temperature : `pint.Quantity`
         Air temperature
-    rh : `pint.Quantity`
+    relative_humidity : `pint.Quantity`
         The relative humidity expressed as a unitless ratio in the range [0, 1].
         Can also pass a percentage if proper units are attached.
 
     Returns
     -------
     `pint.Quantity`
-        The corresponding Heat Index value(s)
+        Corresponding Heat Index value(s)
 
     Other Parameters
     ----------------
@@ -238,31 +244,35 @@ def heat_index(temperature, rh, mask_undefined=True):
         A flag indicating whether a masked array should be returned with
         values masked where the temperature < 80F. Defaults to `True`.
 
+
+    .. versionchanged:: 1.0
+       Renamed ``rh`` parameter to ``relative_humidity``
+
     See Also
     --------
-    windchill
+    windchill, apparent_temperature
 
     """
-    temperature = atleast_1d(temperature)
-    rh = atleast_1d(rh)
-    # assign units to rh if they currently are not present
-    if not hasattr(rh, 'units'):
-        rh = rh * units.dimensionless
+    temperature = np.atleast_1d(temperature)
+    relative_humidity = np.atleast_1d(relative_humidity)
+    # assign units to relative_humidity if they currently are not present
+    if not hasattr(relative_humidity, 'units'):
+        relative_humidity = relative_humidity * units.dimensionless
     delta = temperature.to(units.degF) - 0. * units.degF
-    rh2 = rh * rh
+    rh2 = relative_humidity * relative_humidity
     delta2 = delta * delta
 
-    # Simplifed Heat Index -- constants converted for RH in [0, 1]
-    a = -10.3 * units.degF + 1.1 * delta + 4.7 * units.delta_degF * rh
+    # Simplifed Heat Index -- constants converted for relative_humidity in [0, 1]
+    a = -10.3 * units.degF + 1.1 * delta + 4.7 * units.delta_degF * relative_humidity
 
-    # More refined Heat Index -- constants converted for RH in [0, 1]
+    # More refined Heat Index -- constants converted for relative_humidity in [0, 1]
     b = (-42.379 * units.degF
          + 2.04901523 * delta
-         + 1014.333127 * units.delta_degF * rh
-         - 22.475541 * delta * rh
+         + 1014.333127 * units.delta_degF * relative_humidity
+         - 22.475541 * delta * relative_humidity
          - 6.83783e-3 / units.delta_degF * delta2
          - 5.481717e2 * units.delta_degF * rh2
-         + 1.22874e-1 / units.delta_degF * delta2 * rh
+         + 1.22874e-1 / units.delta_degF * delta2 * relative_humidity
          + 8.5282 * delta * rh2
          - 1.99e-2 / units.delta_degF * delta2 * rh2)
 
@@ -288,20 +298,22 @@ def heat_index(temperature, rh, mask_undefined=True):
         hi[sel] = b[sel]
 
     # Adjustment for RH <= 13% and 80F <= T <= 112F
-    sel = ((rh <= 13. * units.percent) & (temperature >= 80. * units.degF)
+    sel = ((relative_humidity <= 13. * units.percent) & (temperature >= 80. * units.degF)
            & (temperature <= 112. * units.degF))
     if np.any(sel):
-        rh15adj = ((13. - rh * 100.) / 4.
-                   * ((17. * units.delta_degF - np.abs(delta - 95. * units.delta_degF))
-                      / 17. * units.delta_degF) ** 0.5)
-        hi[sel] = hi[sel] - rh15adj[sel]
+        rh15adj = ((13. - relative_humidity[sel] * 100.) / 4.
+                   * np.sqrt((17. * units.delta_degF
+                              - np.abs(delta[sel] - 95. * units.delta_degF))
+                             / 17. * units.delta_degF))
+        hi[sel] = hi[sel] - rh15adj
 
     # Adjustment for RH > 85% and 80F <= T <= 87F
-    sel = ((rh > 85. * units.percent) & (temperature >= 80. * units.degF)
+    sel = ((relative_humidity > 85. * units.percent) & (temperature >= 80. * units.degF)
            & (temperature <= 87. * units.degF))
     if np.any(sel):
-        rh85adj = 0.02 * (rh * 100. - 85.) * (87. * units.delta_degF - delta)
-        hi[sel] = hi[sel] + rh85adj[sel]
+        rh85adj = 0.02 * (relative_humidity[sel] * 100. - 85.) * (87. * units.delta_degF
+                                                                  - delta[sel])
+        hi[sel] = hi[sel] + rh85adj
 
     # See if we need to mask any undefined values
     if mask_undefined:
@@ -312,9 +324,10 @@ def heat_index(temperature, rh, mask_undefined=True):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='temperature')
 @check_units(temperature='[temperature]', speed='[speed]')
-def apparent_temperature(temperature, rh, speed, face_level_winds=False, mask_undefined=True):
+def apparent_temperature(temperature, relative_humidity, speed, face_level_winds=False,
+                         mask_undefined=True):
     r"""Calculate the current apparent temperature.
 
     Calculates the current apparent temperature based on the wind chill or heat index
@@ -323,12 +336,12 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False, mask_un
     Parameters
     ----------
     temperature : `pint.Quantity`
-        The air temperature
-    rh : `pint.Quantity`
-        The relative humidity expressed as a unitless ratio in the range [0, 1].
+        Air temperature
+    relative_humidity : `pint.Quantity`
+        Relative humidity expressed as a unitless ratio in the range [0, 1].
         Can also pass a percentage if proper units are attached.
     speed : `pint.Quantity`
-        The wind speed at 10m.  If instead the winds are at face level,
+        Wind speed at 10m.  If instead the winds are at face level,
         `face_level_winds` should be set to `True` and the 1.5 multiplicative
         correction will be applied automatically.
     face_level_winds : bool, optional
@@ -346,7 +359,11 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False, mask_un
     Returns
     -------
     `pint.Quantity`
-        The corresponding apparent temperature value(s)
+        Corresponding apparent temperature value(s)
+
+
+    .. versionchanged:: 1.0
+       Renamed ``rh`` parameter to ``relative_humidity``
 
     See Also
     --------
@@ -355,15 +372,15 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False, mask_un
     """
     is_not_scalar = isinstance(temperature.m, (list, tuple, np.ndarray))
 
-    temperature = atleast_1d(temperature)
-    rh = atleast_1d(rh)
-    speed = atleast_1d(speed)
+    temperature = np.atleast_1d(temperature)
+    relative_humidity = np.atleast_1d(relative_humidity)
+    speed = np.atleast_1d(speed)
 
     # NB: mask_defined=True is needed to know where computed values exist
     wind_chill_temperature = windchill(temperature, speed, face_level_winds=face_level_winds,
                                        mask_undefined=True).to(temperature.units)
 
-    heat_index_temperature = heat_index(temperature, rh,
+    heat_index_temperature = heat_index(temperature, relative_humidity,
                                         mask_undefined=True).to(temperature.units)
 
     # Combine the heat index and wind chill arrays (no point has a value in both)
@@ -386,14 +403,14 @@ def apparent_temperature(temperature, rh, speed, face_level_winds=False, mask_un
     if is_not_scalar:
         return app_temperature
     else:
-        return atleast_1d(app_temperature)[0]
+        return np.atleast_1d(app_temperature)[0]
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='pressure')
 @check_units('[pressure]')
 def pressure_to_height_std(pressure):
-    r"""Convert pressure data to heights using the U.S. standard atmosphere [NOAA1976]_.
+    r"""Convert pressure data to height using the U.S. standard atmosphere [NOAA1976]_.
 
     The implementation uses the formula outlined in [Hobbs1977]_ pg.60-61.
 
@@ -405,20 +422,19 @@ def pressure_to_height_std(pressure):
     Returns
     -------
     `pint.Quantity`
-        The corresponding height value(s)
+        Corresponding height value(s)
 
     Notes
     -----
     .. math:: Z = \frac{T_0}{\Gamma}[1-\frac{p}{p_0}^\frac{R\Gamma}{g}]
 
     """
-    gamma = 6.5 * units('K/km')
     return (t0 / gamma) * (1 - (pressure / p0).to('dimensionless')**(
         mpconsts.Rd * gamma / mpconsts.g))
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='height')
 @check_units('[length]')
 def height_to_geopotential(height):
     r"""Compute geopotential for a given height above sea level.
@@ -430,7 +446,8 @@ def height_to_geopotential(height):
     .. math:: \Phi = \frac{g R_e z}{R_e + z}
 
     (where :math:`\Phi` is geopotential, :math:`z` is height, :math:`R_e` is average Earth
-    radius, and :math:`g` is standard gravity.)
+    radius, and :math:`g` is standard gravity).
+
 
     Parameters
     ----------
@@ -440,7 +457,7 @@ def height_to_geopotential(height):
     Returns
     -------
     `pint.Quantity`
-        The corresponding geopotential value(s)
+        Corresponding geopotential value(s)
 
     Examples
     --------
@@ -449,9 +466,9 @@ def height_to_geopotential(height):
     >>> height = np.linspace(0, 10000, num=11) * units.m
     >>> geopot = metpy.calc.height_to_geopotential(height)
     >>> geopot
-    <Quantity([     0.           9805.11102602  19607.14506998  29406.10358006
-    39201.98800351  48994.79978671  58784.54037509  68571.21121319
-    78354.81374467  88135.34941224  97912.81965774], 'meter ** 2 / second ** 2')>
+    <Quantity([     0.           9805.11097983 19607.1448853  29406.10316465
+    39201.98726524 48994.79863351 58784.53871501 68571.20895435
+    78354.81079527 88135.34568058 97912.81505219], 'meter ** 2 / second ** 2')>
 
     Notes
     -----
@@ -464,7 +481,7 @@ def height_to_geopotential(height):
     centrifugal force and Earth's eccentricity.
 
     (Prior to MetPy v0.11, this formula instead calculated :math:`g(z)` from Newton's Law of
-    Gravitation assuming a spherical Earth and no centrifugal force effects.)
+    Gravitation assuming a spherical Earth and no centrifugal force effects).
 
     See Also
     --------
@@ -475,8 +492,8 @@ def height_to_geopotential(height):
 
 
 @exporter.export
-@preprocess_xarray
-def geopotential_to_height(geopot):
+@preprocess_and_wrap(wrap_like='geopotential')
+def geopotential_to_height(geopotential):
     r"""Compute height above sea level from a given geopotential.
 
     Calculates the height above mean sea level from geopotential using the following formula,
@@ -486,7 +503,8 @@ def geopotential_to_height(geopot):
     .. math:: z = \frac{\Phi R_e}{gR_e - \Phi}
 
     (where :math:`\Phi` is geopotential, :math:`z` is height, :math:`R_e` is average Earth
-    radius, and :math:`g` is standard gravity.)
+    radius, and :math:`g` is standard gravity).
+
 
     Parameters
     ----------
@@ -496,7 +514,7 @@ def geopotential_to_height(geopot):
     Returns
     -------
     `pint.Quantity`
-        The corresponding value(s) of height above sea level
+        Corresponding value(s) of height above sea level
 
     Examples
     --------
@@ -505,9 +523,9 @@ def geopotential_to_height(geopot):
     >>> height = np.linspace(0, 10000, num=11) * units.m
     >>> geopot = metpy.calc.height_to_geopotential(height)
     >>> geopot
-    <Quantity([     0.           9805.11102602  19607.14506998  29406.10358006
-    39201.98800351  48994.79978671  58784.54037509  68571.21121319
-    78354.81374467  88135.34941224  97912.81965774], 'meter ** 2 / second ** 2')>
+    <Quantity([     0.           9805.11097983 19607.1448853  29406.10316465
+    39201.98726524 48994.79863351 58784.53871501 68571.20895435
+    78354.81079527 88135.34568058 97912.81505219], 'meter ** 2 / second ** 2')>
     >>> height = metpy.calc.geopotential_to_height(geopot)
     >>> height
     <Quantity([     0.   1000.   2000.   3000.   4000.   5000.   6000.   7000.   8000.
@@ -526,16 +544,19 @@ def geopotential_to_height(geopot):
     (Prior to MetPy v0.11, this formula instead calculated :math:`g(z)` from Newton's Law of
     Gravitation assuming a spherical Earth and no centrifugal force effects.)
 
+    .. versionchanged:: 1.0
+       Renamed ``geopot`` parameter to ``geopotential``
+
     See Also
     --------
     height_to_geopotential
 
     """
-    return (geopot * mpconsts.Re) / (mpconsts.g * mpconsts.Re - geopot)
+    return (geopotential * mpconsts.Re) / (mpconsts.g * mpconsts.Re - geopotential)
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='height')
 @check_units('[length]')
 def height_to_pressure_std(height):
     r"""Convert height data to pressures using the U.S. standard atmosphere [NOAA1976]_.
@@ -550,19 +571,18 @@ def height_to_pressure_std(height):
     Returns
     -------
     `pint.Quantity`
-        The corresponding pressure value(s)
+        Corresponding pressure value(s)
 
     Notes
     -----
     .. math:: p = p_0 e^{\frac{g}{R \Gamma} \text{ln}(1-\frac{Z \Gamma}{T_0})}
 
     """
-    gamma = 6.5 * units('K/km')
     return p0 * (1 - (gamma / t0) * height) ** (mpconsts.g / (mpconsts.Rd * gamma))
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='latitude')
 def coriolis_parameter(latitude):
     r"""Calculate the coriolis parameter at each point.
 
@@ -576,7 +596,7 @@ def coriolis_parameter(latitude):
     Returns
     -------
     `pint.Quantity`
-        The corresponding coriolis force at each point
+        Corresponding coriolis force at each point
 
     """
     latitude = _check_radians(latitude, max_radians=np.pi / 2)
@@ -584,7 +604,7 @@ def coriolis_parameter(latitude):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='pressure')
 @check_units('[pressure]', '[length]')
 def add_height_to_pressure(pressure, height):
     r"""Calculate the pressure at a certain height above another pressure level.
@@ -601,7 +621,7 @@ def add_height_to_pressure(pressure, height):
     Returns
     -------
     `pint.Quantity`
-        The corresponding pressure value for the height above the pressure level
+        Corresponding pressure value for the height above the pressure level
 
     See Also
     --------
@@ -613,7 +633,7 @@ def add_height_to_pressure(pressure, height):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='height')
 @check_units('[length]', '[pressure]')
 def add_pressure_to_height(height, pressure):
     r"""Calculate the height at a certain pressure above another height.
@@ -642,30 +662,30 @@ def add_pressure_to_height(height, pressure):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='sigma')
 @check_units('[dimensionless]', '[pressure]', '[pressure]')
-def sigma_to_pressure(sigma, psfc, ptop):
+def sigma_to_pressure(sigma, pressure_sfc, pressure_top):
     r"""Calculate pressure from sigma values.
 
     Parameters
     ----------
     sigma : ndarray
-        The sigma levels to be converted to pressure levels.
+        Sigma levels to be converted to pressure levels
 
-    psfc : `pint.Quantity`
-        The surface pressure value.
+    pressure_sfc : `pint.Quantity`
+        Surface pressure value
 
-    ptop : `pint.Quantity`
-        The pressure value at the top of the model domain.
+    pressure_top : `pint.Quantity`
+        Pressure value at the top of the model domain
 
     Returns
     -------
     `pint.Quantity`
-        The pressure values at the given sigma levels.
+        Pressure values at the given sigma levels
 
     Notes
     -----
-    Sigma definition adapted from [Philips1957]_.
+    Sigma definition adapted from [Philips1957]_:
 
     .. math:: p = \sigma * (p_{sfc} - p_{top}) + p_{top}
 
@@ -674,19 +694,21 @@ def sigma_to_pressure(sigma, psfc, ptop):
     * :math:`p_{sfc}` is pressure at the surface or model floor
     * :math:`p_{top}` is pressure at the top of the model domain
 
+    .. versionchanged:: 1.0
+       Renamed ``psfc``, ``ptop`` parameters to ``pressure_sfc``, ``pressure_top``
+
     """
     if np.any(sigma < 0) or np.any(sigma > 1):
         raise ValueError('Sigma values should be bounded by 0 and 1')
 
-    if psfc.magnitude < 0 or ptop.magnitude < 0:
+    if pressure_sfc.magnitude < 0 or pressure_top.magnitude < 0:
         raise ValueError('Pressure input should be non-negative')
 
-    return sigma * (psfc - ptop) + ptop
+    return sigma * (pressure_sfc - pressure_top) + pressure_top
 
 
 @exporter.export
-@preprocess_xarray
-@units.wraps('=A', ('=A', None))
+@preprocess_and_wrap(wrap_like='scalar_grid', match_unit=True, to_magnitude=True)
 def smooth_gaussian(scalar_grid, n):
     """Filter with normal distribution of weights.
 
@@ -706,7 +728,7 @@ def smooth_gaussian(scalar_grid, n):
 
     Notes
     -----
-    This function is a close replication of the GEMPAK function GWFS,
+    This function is a close replication of the GEMPAK function ``GWFS``,
     but is not identical.  The following notes are incorporated from
     the GEMPAK source code:
 
@@ -714,9 +736,9 @@ def smooth_gaussian(scalar_grid, n):
     low-pass filter whose weights are determined by the normal
     (Gaussian) probability distribution function for two dimensions.
     The weight given to any grid point within the area covered by the
-    moving average for a target grid point is proportional to
+    moving average for a target grid point is proportional to:
 
-                    EXP [ -( D ** 2 ) ],
+    .. math:: e^{-D^2}
 
     where D is the distance from that point to the target point divided
     by the standard deviation of the normal distribution.  The value of
@@ -778,15 +800,189 @@ def smooth_gaussian(scalar_grid, n):
 
 
 @exporter.export
-@preprocess_xarray
-@units.wraps('=A', ('=A', None, None))
+@preprocess_and_wrap(wrap_like='scalar_grid', match_unit=True, to_magnitude=True)
+def smooth_window(scalar_grid, window, passes=1, normalize_weights=True):
+    """Filter with an arbitrary window smoother.
+
+    Parameters
+    ----------
+    scalar_grid : array-like
+        N-dimensional scalar grid to be smoothed
+
+    window : ndarray
+        Window to use in smoothing. Can have dimension less than or equal to N. If
+        dimension less than N, the scalar grid will be smoothed along its trailing dimensions.
+        Shape along each dimension must be odd.
+
+    passes : int
+        The number of times to apply the filter to the grid. Defaults to 1.
+
+    normalize_weights : bool
+        If true, divide the values in window by the sum of all values in the window to obtain
+        the normalized smoothing weights. If false, use supplied values directly as the
+        weights.
+
+    Returns
+    -------
+    array-like
+        The filtered scalar grid
+
+    Notes
+    -----
+    This function can be applied multiple times to create a more smoothed field and will only
+    smooth the interior points, leaving the end points with their original values (this
+    function will leave an unsmoothed edge of size `(n - 1) / 2` for each `n` in the shape of
+    `window` around the data). If a masked value or NaN values exists in the array, it will
+    propagate to any point that uses that particular grid point in the smoothing calculation.
+    Applying the smoothing function multiple times will propogate NaNs further throughout the
+    domain.
+
+    See Also
+    --------
+    smooth_rectangular, smooth_circular, smooth_n_point, smooth_gaussian
+
+    """
+    def _pad(n):
+        # Return number of entries to pad given length along dimension.
+        return (n - 1) // 2
+
+    def _zero_to_none(x):
+        # Convert zero values to None, otherwise return what is given.
+        return x if x != 0 else None
+
+    def _offset(pad, k):
+        # Return padded slice offset by k entries
+        return slice(_zero_to_none(pad + k), _zero_to_none(-pad + k))
+
+    def _trailing_dims(indexer):
+        # Add ... to the front of an indexer, since we are working with trailing dimensions.
+        return (Ellipsis,) + tuple(indexer)
+
+    # Verify that shape in all dimensions is odd (need to have a neighboorhood around a
+    # central point)
+    if any((size % 2 == 0) for size in window.shape):
+        raise ValueError('The shape of the smoothing window must be odd in all dimensions.')
+
+    # Optionally normalize the supplied weighting window
+    if normalize_weights:
+        weights = window / np.sum(window)
+    else:
+        weights = window
+
+    # Set indexes
+    # Inner index for the centered array elements that are affected by the smoothing
+    inner_full_index = _trailing_dims(_offset(_pad(n), 0) for n in weights.shape)
+    # Indexes to iterate over each weight
+    weight_indexes = tuple(product(*(range(n) for n in weights.shape)))
+
+    # Index for full array elements, offset by the weight index
+    def offset_full_index(weight_index):
+        return _trailing_dims(_offset(_pad(n), weight_index[i] - _pad(n))
+                              for i, n in enumerate(weights.shape))
+
+    # TODO: this is not lazy-loading/dask compatible, as it "densifies" the data
+    data = np.array(scalar_grid)
+    for _ in range(passes):
+        # Set values corresponding to smoothing weights by summing over each weight and
+        # applying offsets in needed dimensions
+        data[inner_full_index] = sum(weights[index] * data[offset_full_index(index)]
+                                     for index in weight_indexes)
+
+    return data
+
+
+@exporter.export
+def smooth_rectangular(scalar_grid, size, passes=1):
+    """Filter with a rectangular window smoother.
+
+    Parameters
+    ----------
+    scalar_grid : array-like
+        N-dimensional scalar grid to be smoothed
+
+    size : int or sequence of ints
+        Shape of rectangle along the trailing dimension(s) of the scalar grid
+
+    passes : int
+        The number of times to apply the filter to the grid. Defaults to 1.
+
+    Returns
+    -------
+    array-like
+        The filtered scalar grid
+
+    Notes
+    -----
+    This function can be applied multiple times to create a more smoothed field and will only
+    smooth the interior points, leaving the end points with their original values (this
+    function will leave an unsmoothed edge of size `(n - 1) / 2` for each `n` in `size` around
+    the data). If a masked value or NaN values exists in the array, it will propagate to any
+    point that uses that particular grid point in the smoothing calculation. Applying the
+    smoothing function multiple times will propogate NaNs further throughout the domain.
+
+    See Also
+    --------
+    smooth_window, smooth_circular, smooth_n_point, smooth_gaussian
+
+    """
+    return smooth_window(scalar_grid, np.ones(size), passes=passes)
+
+
+@exporter.export
+def smooth_circular(scalar_grid, radius, passes=1):
+    """Filter with a circular window smoother.
+
+    Parameters
+    ----------
+    scalar_grid : array-like
+        N-dimensional scalar grid to be smoothed. If more than two axes, smoothing is only
+        done along the last two.
+
+    radius : int
+        Radius of the circular smoothing window. The "diameter" of the circle (width of
+        smoothing window) is 2 * radius + 1 to provide a smoothing window with odd shape.
+
+    passes : int
+        The number of times to apply the filter to the grid. Defaults to 1.
+
+    Returns
+    -------
+    array-like
+        The filtered scalar grid
+
+    Notes
+    -----
+    This function can be applied multiple times to create a more smoothed field and will only
+    smooth the interior points, leaving the end points with their original values (this
+    function will leave an unsmoothed edge of size `radius` around the data). If a masked
+    value or NaN values exists in the array, it will propagate to any point that uses that
+    particular grid point in the smoothing calculation. Applying the smoothing function
+    multiple times will propogate NaNs further throughout the domain.
+
+    See Also
+    --------
+    smooth_window, smooth_rectangular, smooth_n_point, smooth_gaussian
+
+    """
+    # Generate the circle
+    size = 2 * radius + 1
+    x, y = np.mgrid[:size, :size]
+    distance = np.sqrt((x - radius) ** 2 + (y - radius) ** 2)
+    circle = distance <= radius
+
+    # Apply smoother
+    return smooth_window(scalar_grid, circle, passes=passes)
+
+
+@exporter.export
 def smooth_n_point(scalar_grid, n=5, passes=1):
-    """Filter with normal distribution of weights.
+    """Filter with an n-point smoother.
 
     Parameters
     ----------
     scalar_grid : array-like or `pint.Quantity`
-        Some 2D scalar grid to be smoothed.
+        N-dimensional scalar grid to be smoothed. If more than two axes, smoothing is only
+        done along the last two.
 
     n: int
         The number of points to use in smoothing, only valid inputs
@@ -798,45 +994,41 @@ def smooth_n_point(scalar_grid, n=5, passes=1):
     Returns
     -------
     array-like or `pint.Quantity`
-        The filtered 2D scalar grid.
+        The filtered scalar grid
 
     Notes
     -----
-    This function is a close replication of the GEMPAK function SM5S
-    and SM9S depending on the choice of the number of points to use
-    for smoothing. This function can be applied multiple times to
-    create a more smoothed field and will only smooth the interior
-    points, leaving the end points with their original values. If a
-    masked value or NaN values exists in the array, it will propagate
-    to any point that uses that particular grid point in the smoothing
-    calculation. Applying the smoothing function multiple times will
-    propagate NaNs further throughout the domain.
+    This function is a close replication of the GEMPAK function SM5S and SM9S depending on the
+    choice of the number of points to use for smoothing. This function can be applied multiple
+    times to create a more smoothed field and will only smooth the interior points, leaving
+    the end points with their original values (this function will leave an unsmoothed edge of
+    size 1 around the data). If a masked value or NaN values exists in the array, it will
+    propagate to any point that uses that particular grid point in the smoothing calculation.
+    Applying the smoothing function multiple times will propogate NaNs further throughout the
+    domain.
+
+    See Also
+    --------
+    smooth_window, smooth_rectangular, smooth_circular, smooth_gaussian
 
     """
     if n == 9:
-        p = 0.25
-        q = 0.125
-        r = 0.0625
+        weights = np.array([[0.0625, 0.125, 0.0625],
+                            [0.125, 0.25, 0.125],
+                            [0.0625, 0.125, 0.0625]])
     elif n == 5:
-        p = 0.5
-        q = 0.125
-        r = 0.0
+        weights = np.array([[0., 0.125, 0.],
+                            [0.125, 0.5, 0.125],
+                            [0., 0.125, 0.]])
     else:
         raise ValueError('The number of points to use in the smoothing '
                          'calculation must be either 5 or 9.')
 
-    smooth_grid = scalar_grid[:].copy()
-    for _i in range(passes):
-        smooth_grid[1:-1, 1:-1] = (p * smooth_grid[1:-1, 1:-1]
-                                   + q * (smooth_grid[2:, 1:-1] + smooth_grid[1:-1, 2:]
-                                          + smooth_grid[:-2, 1:-1] + smooth_grid[1:-1, :-2])
-                                   + r * (smooth_grid[2:, 2:] + smooth_grid[2:, :-2] +
-                                          + smooth_grid[:-2, 2:] + smooth_grid[:-2, :-2]))
-    return smooth_grid
+    return smooth_window(scalar_grid, window=weights, passes=passes, normalize_weights=False)
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='altimeter_value')
 @check_units('[pressure]', '[length]')
 def altimeter_to_station_pressure(altimeter_value, height):
     r"""Convert the altimeter measurement to station pressure.
@@ -857,14 +1049,15 @@ def altimeter_to_station_pressure(altimeter_value, height):
     altimeter_value : `pint.Quantity`
         The altimeter setting value as defined by the METAR or other observation,
         which can be measured in either inches of mercury (in. Hg) or millibars (mb)
+
     height: `pint.Quantity`
-        Elevation of the station measuring pressure.
+        Elevation of the station measuring pressure
 
     Returns
     -------
     `pint.Quantity`
-        The station pressure in hPa or in. Hg, which can be used to calculate sea-level
-        pressure
+        The station pressure in hPa or in. Hg. Can be used to calculate sea-level
+        pressure.
 
     See Also
     --------
@@ -882,7 +1075,7 @@ def altimeter_to_station_pressure(altimeter_value, height):
      .. math::  F = \left [1 + \left(\frac{p_{0}^n a}{T_{0}} \right)
                    \frac{H_{b}}{p_{1}^n} \right ] ^ \frac{1}{n}
 
-    Where
+    Where,
 
     :math:`p_{0}` = standard sea-level pressure = 1013.25 mb
 
@@ -893,11 +1086,10 @@ def altimeter_to_station_pressure(altimeter_value, height):
 
     :math:`t_{0}` = standard sea-level temperature 288 K
 
-    :math:`H_{b} =` station elevation in meters (elevation for which station
-      pressure is given)
+    :math:`H_{b} =` station elevation in meters (elevation for which station pressure is given)
 
-    :math:`n = \frac{a R_{d}}{g} = 0.190284` where :math:`R_{d}` is the gas
-      constant for dry air
+    :math:`n = \frac{a R_{d}}{g} = 0.190284` where :math:`R_{d}` is the gas constant for dry
+    air
 
     And solving for :math:`p_{mb}` results in the equation below, which is used to
     calculate station pressure :math:`(p_{mb})`
@@ -906,9 +1098,6 @@ def altimeter_to_station_pressure(altimeter_value, height):
                        \right) \right] ^ \frac{1}{n} + 0.3
 
     """
-    # Gamma Value for this case
-    gamma = 0.0065 * units('K/m')
-
     # N-Value
     n = (mpconsts.Rd * gamma / mpconsts.g).to_base_units()
 
@@ -918,7 +1107,7 @@ def altimeter_to_station_pressure(altimeter_value, height):
 
 
 @exporter.export
-@preprocess_xarray
+@preprocess_and_wrap(wrap_like='altimeter_value')
 @check_units('[pressure]', '[length]', '[temperature]')
 def altimeter_to_sea_level_pressure(altimeter_value, height, temperature):
     r"""Convert the altimeter setting to sea-level pressure.
@@ -926,7 +1115,7 @@ def altimeter_to_sea_level_pressure(altimeter_value, height, temperature):
     This function is useful for working with METARs since most provide
     altimeter values, but not sea-level pressure, which is often plotted
     on surface maps. The following definitions of altimeter setting, station pressure, and
-    sea-level pressure are taken from [Smithsonian1951]_
+    sea-level pressure are taken from [Smithsonian1951]_.
     Altimeter setting is the pressure value to which an aircraft altimeter scale
     is set so that it will indicate the altitude above mean sea-level of an aircraft
     on the ground at the location for which the value is determined. It assumes a standard
@@ -942,9 +1131,11 @@ def altimeter_to_sea_level_pressure(altimeter_value, height, temperature):
     ----------
     altimeter_value : 'pint.Quantity'
         The altimeter setting value is defined by the METAR or other observation,
-        with units of inches of mercury (in Hg) or millibars (hPa)
+        with units of inches of mercury (in Hg) or millibars (hPa).
+
     height  : 'pint.Quantity'
         Elevation of the station measuring pressure. Often times measured in meters
+
     temperature : 'pint.Quantity'
         Temperature at the station
 
@@ -952,7 +1143,7 @@ def altimeter_to_sea_level_pressure(altimeter_value, height, temperature):
     -------
     'pint.Quantity'
         The sea-level pressure in hPa and makes pressure values easier to compare
-        between different stations
+        between different stations.
 
     See Also
     --------
@@ -960,7 +1151,7 @@ def altimeter_to_sea_level_pressure(altimeter_value, height, temperature):
 
     Notes
     -----
-    This function is implemented using the following equations from Wallace and Hobbs (1977)
+    This function is implemented using the following equations from Wallace and Hobbs (1977).
 
     Equation 2.29:
      .. math::
@@ -994,22 +1185,20 @@ def _check_radians(value, max_radians=2 * np.pi):
     Parameters
     ----------
     value : `pint.Quantity`
-        The input value to check.
+        Input value to check
 
     max_radians : float
-        Maximum absolute value of radians before warning.
+        Maximum absolute value of radians before warning
 
     Returns
     -------
     `pint.Quantity`
-        The input value
+        Input value
 
     """
-    try:
+    with contextlib.suppress(AttributeError):
         value = value.to('radians').m
-    except AttributeError:
-        pass
-    if np.greater(np.nanmax(np.abs(value)), max_radians):
+    if np.any(np.greater(np.abs(value), max_radians)):
         warnings.warn('Input over {} radians. '
-                      'Ensure proper units are given.'.format(max_radians))
+                      'Ensure proper units are given.'.format(np.nanmax(max_radians)))
     return value
