@@ -16,13 +16,10 @@ from .station_data import station_info
 from ..calc import altimeter_to_sea_level_pressure, wind_components
 from ..package_tools import Exporter
 from ..plots.wx_symbols import wx_code_map
-from ..units import pandas_dataframe_to_unit_arrays, units
+from ..units import units
 
 exporter = Exporter(globals())
 
-# Ignore the pandas warning
-warnings.filterwarnings('ignore', "Pandas doesn't allow columns to be created", UserWarning)
-
 # Configure the named tuple used for storing METAR data
 Metar = namedtuple('metar', ['station_id', 'latitude', 'longitude', 'elevation',
                              'date_time', 'wind_direction', 'wind_speed', 'current_wx1',
@@ -64,7 +61,7 @@ col_units = {'station_id': None,
 
 
 @exporter.export
-def parse_metar_to_dataframe(metar_text, year=datetime.now().year, month=datetime.now().month):
+def parse_metar_to_dataframe(metar_text, *, year=None, month=None):
     """Parse a single METAR report into a Pandas DataFrame.
 
     Takes a METAR string in a text form, and creates a `pandas.DataFrame` including the
@@ -79,9 +76,9 @@ def parse_metar_to_dataframe(metar_text, year=datetime.now().year, month=datetim
     metar_text : str
         The METAR report
     year : int, optional
-        Year in which observation was taken, defaults to the current year
+        Year in which observation was taken, defaults to current year. Keyword-only argument.
     month : int, optional
-        Month in which observation was taken, defaults to the current month
+        Month in which observation was taken, defaults to current month. Keyword-only argument.
 
     Returns
     -------
@@ -119,6 +116,12 @@ def parse_metar_to_dataframe(metar_text, year=datetime.now().year, month=datetim
     and altimeter value, float
 
     """
+    # Defaults year and/or month to present reported date if not provided
+    if year is None or month is None:
+        now = datetime.now()
+        year = now.year if year is None else year
+        month = now.month if month is None else month
+
     # Use the named tuple parsing function to separate metar
     # Utilizes the station dictionary which contains elevation, latitude, and longitude
     metar_vars = parse_metar_to_named_tuple(metar_text, station_info, year, month)
@@ -171,18 +174,15 @@ def parse_metar_to_dataframe(metar_text, year=datetime.now().year, month=datetim
     df['altimeter'] = df.altimeter.round(2)
     df['air_pressure_at_sea_level'] = df.air_pressure_at_sea_level.round(2)
 
-    # Set the units for the dataframe
-    df.units = col_units
+    # Set the units for the dataframe--filter out warning from pandas
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', UserWarning)
+        df.units = col_units
 
-    # Add the array for units to the dataframe
-    pandas_dataframe_to_unit_arrays(df)
-
-    # Return the dataframe
     return df
 
 
-def parse_metar_to_named_tuple(metar_text, station_metadata, year=datetime.now().year,
-                               month=datetime.now().month):
+def parse_metar_to_named_tuple(metar_text, station_metadata, year, month):
     """Parse a METAR report in text form into a list of named tuples.
 
     Parameters
@@ -191,10 +191,15 @@ def parse_metar_to_named_tuple(metar_text, station_metadata, year=datetime.now()
         The METAR report
     station_metadata : dict
         Mapping of station identifiers to station metadata
+    year : int
+        Reported year of observation for constructing 'date_time'
+    month : int
+        Reported month of observation for constructing 'date_time'
 
     Returns
     -------
-    `pandas.DataFrame`
+    metar : namedtuple
+        Named tuple of parsed METAR fields
 
     Notes
     -----
@@ -429,7 +434,7 @@ def parse_metar_to_named_tuple(metar_text, station_metadata, year=datetime.now()
 
 
 @exporter.export
-def parse_metar_file(filename, year=datetime.now().year, month=datetime.now().month):
+def parse_metar_file(filename, *, year=None, month=None):
     """Parse a text file containing multiple METAR reports and/or text products.
 
     Parameters
@@ -438,9 +443,9 @@ def parse_metar_file(filename, year=datetime.now().year, month=datetime.now().mo
         If str, the name of the file to be opened. If `filename` is a file-like object,
         this will be read from directly.
     year : int, optional
-        Year in which observation was taken, defaults to the current year
+        Year in which observation was taken, defaults to current year. Keyword-only argument.
     month : int, optional
-        Month in which observation was taken, defaults to the current month
+        Month in which observation was taken, defaults to current month. Keyword-only argument.
 
     Returns
     -------
@@ -478,6 +483,12 @@ def parse_metar_file(filename, year=datetime.now().year, month=datetime.now().mo
     and altimeter value, float
 
     """
+    # Defaults year and/or month to present reported date if not provided
+    if year is None or month is None:
+        now = datetime.now()
+        year = now.year if year is None else year
+        month = now.month if month is None else month
+
     # Function to merge METARs
     def merge(x, key='     '):
         tmp = []
@@ -623,8 +634,9 @@ def parse_metar_file(filename, year=datetime.now().year, month=datetime.now().mo
     df['altimeter'] = df.altimeter.round(2)
     df['air_pressure_at_sea_level'] = df.air_pressure_at_sea_level.round(2)
 
-    # Set the units for the dataframe
-    df.units = col_units
-    pandas_dataframe_to_unit_arrays(df)
+    # Set the units for the dataframe--filter out warning from Pandas
+    with warnings.catch_warnings():
+        warnings.simplefilter('ignore', UserWarning)
+        df.units = col_units
 
     return df
