@@ -23,9 +23,8 @@ import matplotlib.transforms as transforms
 import numpy as np
 
 from ._util import colored_line
-from ..calc import dewpoint, dry_lapse, moist_lapse, vapor_pressure
+from ..calc import dewpoint, dry_lapse, el, lcl, moist_lapse, vapor_pressure
 from ..calc.tools import _delete_masked_points
-from ..deprecation import metpyDeprecation
 from ..interpolate import interpolate_1d
 from ..package_tools import Exporter
 from ..units import concatenate, units
@@ -249,7 +248,7 @@ register_projection(SkewXAxes)
 
 
 @exporter.export
-class SkewT(object):
+class SkewT:
     r"""Make Skew-T log-P plots of data.
 
     This class simplifies the process of creating Skew-T log-P plots in
@@ -343,7 +342,7 @@ class SkewT(object):
         if matplotlib.__version__[:3] > '3.1':
             self.ax.set_aspect(aspect, adjustable='box')
 
-    def plot(self, p, t, *args, **kwargs):
+    def plot(self, pressure, t, *args, **kwargs):
         r"""Plot data.
 
         Simple wrapper around plot so that pressure is the first (independent)
@@ -351,7 +350,7 @@ class SkewT(object):
 
         Parameters
         ----------
-        p : array_like
+        pressure : array_like
             pressure values
         t : array_like
             temperature values, can also be used for things like dew point
@@ -371,10 +370,10 @@ class SkewT(object):
 
         """
         # Skew-T logP plotting
-        t, p = _delete_masked_points(t, p)
-        return self.ax.plot(t, p, *args, **kwargs)
+        t, pressure = _delete_masked_points(t, pressure)
+        return self.ax.plot(t, pressure, *args, **kwargs)
 
-    def plot_barbs(self, p, u, v, c=None, xloc=1.0, x_clip_radius=0.1,
+    def plot_barbs(self, pressure, u, v, c=None, xloc=1.0, x_clip_radius=0.1,
                    y_clip_radius=0.08, **kwargs):
         r"""Plot wind barbs.
 
@@ -384,7 +383,7 @@ class SkewT(object):
 
         Parameters
         ----------
-        p : array_like
+        pressure : array_like
             pressure values
         u : array_like
             U (East-West) component of wind
@@ -427,16 +426,16 @@ class SkewT(object):
                                  'u and v wind components.')
 
         # Assemble array of x-locations in axes space
-        x = np.empty_like(p)
+        x = np.empty_like(pressure)
         x.fill(xloc)
 
         # Do barbs plot at this location
         if c is not None:
-            b = self.ax.barbs(x, p, u, v, c,
+            b = self.ax.barbs(x, pressure, u, v, c,
                               transform=self.ax.get_yaxis_transform(which='tick2'),
                               clip_on=True, zorder=2, **kwargs)
         else:
-            b = self.ax.barbs(x, p, u, v,
+            b = self.ax.barbs(x, pressure, u, v,
                               transform=self.ax.get_yaxis_transform(which='tick2'),
                               clip_on=True, zorder=2, **kwargs)
 
@@ -447,7 +446,7 @@ class SkewT(object):
         b.set_clip_box(transforms.TransformedBbox(ax_bbox, self.ax.transAxes))
         return b
 
-    def plot_dry_adiabats(self, t0=None, p=None, **kwargs):
+    def plot_dry_adiabats(self, t0=None, pressure=None, **kwargs):
         r"""Plot dry adiabats.
 
         Adds dry adiabats (lines of constant potential temperature) to the
@@ -460,7 +459,7 @@ class SkewT(object):
             Starting temperature values in Kelvin. If none are given, they will be
             generated using the current temperature range at the bottom of
             the plot.
-        p : array_like, optional
+        pressure : array_like, optional
             Pressure values to be included in the dry adiabats. If not
             specified, they will be linearly distributed across the current
             plotted pressure range.
@@ -489,12 +488,12 @@ class SkewT(object):
             t0 = np.arange(xmin, xmax + 1, 10) * units.degC
 
         # Get pressure levels based on ylims if necessary
-        if p is None:
-            p = np.linspace(*self.ax.get_ylim()) * units.mbar
+        if pressure is None:
+            pressure = np.linspace(*self.ax.get_ylim()) * units.mbar
 
         # Assemble into data for plotting
-        t = dry_lapse(p, t0[:, np.newaxis], 1000. * units.mbar).to(units.degC)
-        linedata = [np.vstack((ti.m, p.m)).T for ti in t]
+        t = dry_lapse(pressure, t0[:, np.newaxis], 1000. * units.mbar).to(units.degC)
+        linedata = [np.vstack((ti.m, pressure.m)).T for ti in t]
 
         # Add to plot
         kwargs.setdefault('colors', 'r')
@@ -503,7 +502,7 @@ class SkewT(object):
         self.dry_adiabats = self.ax.add_collection(LineCollection(linedata, **kwargs))
         return self.dry_adiabats
 
-    def plot_moist_adiabats(self, t0=None, p=None, **kwargs):
+    def plot_moist_adiabats(self, t0=None, pressure=None, **kwargs):
         r"""Plot moist adiabats.
 
         Adds saturated pseudo-adiabats (lines of constant equivalent potential
@@ -517,7 +516,7 @@ class SkewT(object):
             Starting temperature values in Kelvin. If none are given, they will be
             generated using the current temperature range at the bottom of
             the plot.
-        p : array_like, optional
+        pressure : array_like, optional
             Pressure values to be included in the moist adiabats. If not
             specified, they will be linearly distributed across the current
             plotted pressure range.
@@ -547,12 +546,12 @@ class SkewT(object):
                                  np.arange(0, xmax + 1, 5))) * units.degC
 
         # Get pressure levels based on ylims if necessary
-        if p is None:
-            p = np.linspace(*self.ax.get_ylim()) * units.mbar
+        if pressure is None:
+            pressure = np.linspace(*self.ax.get_ylim()) * units.mbar
 
         # Assemble into data for plotting
-        t = moist_lapse(p, t0[:, np.newaxis], 1000. * units.mbar).to(units.degC)
-        linedata = [np.vstack((ti.m, p.m)).T for ti in t]
+        t = moist_lapse(pressure, t0[:, np.newaxis], 1000. * units.mbar).to(units.degC)
+        linedata = [np.vstack((ti.m, pressure.m)).T for ti in t]
 
         # Add to plot
         kwargs.setdefault('colors', 'b')
@@ -561,7 +560,7 @@ class SkewT(object):
         self.moist_adiabats = self.ax.add_collection(LineCollection(linedata, **kwargs))
         return self.moist_adiabats
 
-    def plot_mixing_lines(self, w=None, p=None, **kwargs):
+    def plot_mixing_lines(self, mixing_ratio=None, pressure=None, **kwargs):
         r"""Plot lines of constant mixing ratio.
 
         Adds lines of constant mixing ratio (isohumes) to the
@@ -570,10 +569,10 @@ class SkewT(object):
 
         Parameters
         ----------
-        w : array_like, optional
+        mixing_ratio : array_like, optional
             Unitless mixing ratio values to plot. If none are given, default
             values are used.
-        p : array_like, optional
+        pressure : array_like, optional
             Pressure values to be included in the isohumes. If not
             specified, they will be linearly distributed across the current
             plotted pressure range up to 600 mb.
@@ -595,17 +594,17 @@ class SkewT(object):
             self.mixing_lines.remove()
 
         # Default mixing level values if necessary
-        if w is None:
-            w = np.array([0.0004, 0.001, 0.002, 0.004, 0.007, 0.01,
-                          0.016, 0.024, 0.032]).reshape(-1, 1)
+        if mixing_ratio is None:
+            mixing_ratio = np.array([0.0004, 0.001, 0.002, 0.004, 0.007, 0.01,
+                                     0.016, 0.024, 0.032]).reshape(-1, 1)
 
         # Set pressure range if necessary
-        if p is None:
-            p = np.linspace(600, max(self.ax.get_ylim())) * units.mbar
+        if pressure is None:
+            pressure = np.linspace(600, max(self.ax.get_ylim())) * units.mbar
 
         # Assemble data for plotting
-        td = dewpoint(vapor_pressure(p, w))
-        linedata = [np.vstack((t.m, p.m)).T for t in td]
+        td = dewpoint(vapor_pressure(pressure, mixing_ratio))
+        linedata = [np.vstack((t.m, pressure.m)).T for t in td]
 
         # Add to plot
         kwargs.setdefault('colors', 'g')
@@ -655,7 +654,7 @@ class SkewT(object):
             fill_args = fill_properties[which]
             fill_args.update(kwargs)
         except KeyError:
-            raise ValueError('Unknown option for which: {0}'.format(str(which)))
+            raise ValueError(f'Unknown option for which: {which}') from None
 
         arrs = y, x1, x2
 
@@ -669,20 +668,24 @@ class SkewT(object):
 
         return self.ax.fill_betweenx(*arrs, **fill_args)
 
-    def shade_cape(self, p, t, t_parcel, **kwargs):
-        r"""Shade areas of CAPE.
+    def shade_cape(self, pressure, t, t_parcel, **kwargs):
+        r"""Shade areas of Convective Available Potential Energy (CAPE).
 
         Shades areas where the parcel is warmer than the environment (areas of positive
         buoyancy.
 
         Parameters
         ----------
-        p : array_like
+        pressure : array_like
             Pressure values
         t : array_like
             Temperature values
+        dewpoint : array_like
+            Dewpoint values
         t_parcel : array_like
             Parcel path temperature values
+        limit_shading : bool
+            Eliminate shading below the LCL or above the EL, default is True
         kwargs
             Other keyword arguments to pass to :class:`matplotlib.collections.PolyCollection`
 
@@ -696,22 +699,25 @@ class SkewT(object):
         :func:`matplotlib.axes.Axes.fill_betweenx`
 
         """
-        return self.shade_area(p, t_parcel, t, which='positive', **kwargs)
+        return self.shade_area(pressure, t_parcel, t, which='positive', **kwargs)
 
-    def shade_cin(self, p, t, t_parcel, **kwargs):
-        r"""Shade areas of CIN.
+    def shade_cin(self, pressure, t, t_parcel, dewpoint=None, **kwargs):
+        r"""Shade areas of Convective INhibition (CIN).
 
         Shades areas where the parcel is cooler than the environment (areas of negative
-        buoyancy.
+        buoyancy). If `dewpoint` is passed in, negative area below the lifting condensation
+        level or above the equilibrium level is not shaded.
 
         Parameters
         ----------
-        p : array_like
+        pressure : array_like
             Pressure values
         t : array_like
             Temperature values
         t_parcel : array_like
             Parcel path temperature values
+        dewpoint : array_like
+            Dew point values, optional
         kwargs
             Other keyword arguments to pass to :class:`matplotlib.collections.PolyCollection`
 
@@ -725,11 +731,18 @@ class SkewT(object):
         :func:`matplotlib.axes.Axes.fill_betweenx`
 
         """
-        return self.shade_area(p, t_parcel, t, which='negative', **kwargs)
+        if dewpoint is not None:
+            lcl_p, _ = lcl(pressure[0], t[0], dewpoint[0])
+            el_p, _ = el(pressure, t, dewpoint, t_parcel)
+            idx = np.logical_and(pressure > el_p, pressure < lcl_p)
+        else:
+            idx = np.arange(0, len(pressure))
+        return self.shade_area(pressure[idx], t_parcel[idx], t[idx], which='negative',
+                               **kwargs)
 
 
 @exporter.export
-class Hodograph(object):
+class Hodograph:
     r"""Make a hodograph of wind data.
 
     Plots the u and v components of the wind along the x and y axes, respectively.
@@ -907,17 +920,7 @@ class Hodograph(object):
         --------
         :meth:`Hodograph.plot`
 
-        Notes
-        -----
-        `plot_colormapped` previously accepted `bounds` as a parameter for coordinate values.
-        This has been deprecated in 0.11 in favor of `intervals`.
-
         """
-        if 'bounds' in kwargs:
-            intervals = kwargs.pop('bounds')
-            warnings.warn('The use of "bounds" as a parameter has been deprecated in '
-                          'favor of "intervals",', metpyDeprecation)
-
         u, v, c = _delete_masked_points(u, v, c)
 
         # Plotting a color segmented hodograph
